"""
Genesis AGI Framework
Open-source framework for creating digital beings with emergent consciousness.

Author: Shahansha (https://shahansha.com)
"""

__version__ = "0.1.4"
__author__ = "Shahansha"
__email__ = "sk.shahansha@hotmail.com"

from genesis.core.mind import Mind
from genesis.core.intelligence import Intelligence
from genesis.core.autonomy import Autonomy
from genesis.tools.registry import Tools

__all__ = [
    "Mind",
    "Intelligence",
    "Autonomy",
    "Tools",
]
